@@ -63,6 +63,7 @@ struct ECalBackendExchangePrivate {
 	/* Timezones */
 	GHashTable *timezones;
 	icaltimezone *default_timezone;
+	gboolean is_loaded;
 	CalMode mode;
 };
 
@@ -112,6 +113,9 @@ get_ldap_attribute (ECalBackendSync *backend, EDataCal *cal, char **attribute)
 {
 	d(printf("ecbe_get_ldap_attribute(%p, %p)\n", backend, cal));
 
+	if (!attribute)
+		return GNOME_Evolution_Calendar_InvalidObject;
+	
 	/* This is just a hack for SunONE */
 	*attribute = NULL;
 	return GNOME_Evolution_Calendar_Success;
@@ -170,6 +174,11 @@ load_cache (ECalBackendExchange *cbex, E2kUri *e2kuri)
 	g_free (storage_dir);
 	g_free (mangled_uri);
 
+       /* Check if the cache file is present. If it is not present the account might
+          be newly created one. It will be created while save the cache */
+       if (!g_file_test (cbex->priv->object_cache_file, G_FILE_TEST_EXISTS))
+               return GNOME_Evolution_Calendar_Success;
+
 	vcalcomp = e_cal_util_parse_ics_file (cbex->priv->object_cache_file);
 	if (!vcalcomp)
 		return GNOME_Evolution_Calendar_InvalidObject;
@@ -292,14 +301,13 @@ open_calendar (ECalBackendSync *backend, EDataCal *cal, gboolean only_if_exists,
 	ECalBackendExchange *cbex = E_CAL_BACKEND_EXCHANGE (backend);
 	const char *uristr;
 	ExchangeHierarchy *hier;
-	ExchangeAccount *act;
 	ExchangeAccountResult acresult;
 	const char *prop = PR_ACCESS;
 	E2kHTTPStatus status;
 	ECalBackendSyncStatus load_status;
 	E2kResult *results;
 	E2kUri *euri = NULL;
-	int nresults;
+	int nresults = 0;
 	guint access = 0;
 
 	d(printf("ecbe_open_calendar(%p, %p, %sonly if exists, user=%s, pass=%s)\n", backend, cal, only_if_exists?"":"not ", username?username:"(null)", password?password:"(null)"));
@@ -314,6 +322,7 @@ open_calendar (ECalBackendSync *backend, EDataCal *cal, gboolean only_if_exists,
 
 		d(printf ("ECBE : cal is offline .. load cache\n"));
 
+		
 		cbex->priv->read_only = TRUE;
 		source = e_cal_backend_get_source (E_CAL_BACKEND (cbex));
 		display_contents = e_source_get_property (source, "offline_sync");
@@ -323,33 +332,54 @@ open_calendar (ECalBackendSync *backend, EDataCal *cal, gboolean only_if_exists,
 			return GNOME_Evolution_Calendar_RepositoryOffline;
 		}
 
+		cbex->account = exchange_component_get_account_for_uri (global_exchange_component, NULL);
+
+		if (cbex->account) {
+			
+			exchange_account_set_offline (cbex->account);
+			if (!exchange_account_connect (cbex->account, NULL, &acresult)) {
+				cbex->folder = exchange_account_get_folder (cbex->account, uristr);
+			}
+		}
+		
+		if (cbex->priv->is_loaded) {
+			g_mutex_unlock (cbex->priv->open_lock);
+			return GNOME_Evolution_Calendar_Success;
+		}
+
 		euri = e2k_uri_new (uristr);
 		load_status = load_cache (cbex, euri);
 		e2k_uri_free (euri);
+		
+		if (load_status == GNOME_Evolution_Calendar_Success)
+			cbex->priv->is_loaded = TRUE;
 		g_mutex_unlock (cbex->priv->open_lock);
 		return load_status;
 	}
 		
+	/* What else to check */
+	if (cbex->priv->is_loaded && cbex->account && exchange_account_get_context (cbex->account)) {
+		g_mutex_unlock (cbex->priv->open_lock);
+		return GNOME_Evolution_Calendar_Success;
+	}
+		
 	/* Make sure we have an open connection */
 	/* This steals the ExchangeAccount from ExchangeComponent */
-	act = exchange_component_get_account_for_uri (global_exchange_component, NULL);
-	if (!exchange_account_get_context (act)) {
-		if (!exchange_account_connect (act, password, &acresult)) {
+	cbex->account = exchange_component_get_account_for_uri (global_exchange_component, uristr);
+	if (!cbex->account) {
+		cbex->account = exchange_component_get_account_for_uri (global_exchange_component, NULL);
+		if (!exchange_account_connect (cbex->account, password, &acresult)) {
 			g_mutex_unlock (cbex->priv->open_lock);
 			return GNOME_Evolution_Calendar_AuthenticationFailed;
 		}
 	}
 
-	cbex->account = exchange_component_get_account_for_uri (global_exchange_component, uristr);
-	if (!cbex->account) {
-		g_mutex_unlock (cbex->priv->open_lock);
-		return GNOME_Evolution_Calendar_PermissionDenied;
-	}
-
 	if (!exchange_account_get_context (cbex->account)) {
-		g_mutex_unlock (cbex->priv->open_lock);
-		return GNOME_Evolution_Calendar_RepositoryOffline; 
-	}	
+		if(!exchange_account_connect (cbex->account, password, &acresult)) {
+			g_mutex_unlock (cbex->priv->open_lock);
+			return GNOME_Evolution_Calendar_AuthenticationFailed;
+		}
+	}
 
 	cbex->folder = exchange_account_get_folder (cbex->account, uristr);
 	if (!cbex->folder) {
@@ -380,6 +410,8 @@ open_calendar (ECalBackendSync *backend, EDataCal *cal, gboolean only_if_exists,
 
 	if (!(access & MAPI_ACCESS_READ)) {
 		g_mutex_unlock (cbex->priv->open_lock);
+		if (nresults)
+			e2k_results_free (results, nresults);
 		return GNOME_Evolution_Calendar_PermissionDenied;
 	}
 
@@ -387,8 +419,13 @@ open_calendar (ECalBackendSync *backend, EDataCal *cal, gboolean only_if_exists,
 
 	load_status = load_cache (cbex, euri);
 
+	if (load_status == GNOME_Evolution_Calendar_Success)
+		cbex->priv->is_loaded = TRUE;
+
 	g_mutex_unlock (cbex->priv->open_lock);
 	
+	if (nresults)
+		e2k_results_free (results, nresults);
 	return load_status;
 }
 
@@ -467,8 +504,23 @@ static void
 uncache (gpointer key, gpointer value, gpointer data)
 {
 	ECalBackendExchange *cbex = data;
+	ECalBackend *backend = E_CAL_BACKEND (cbex);
+	ECalComponentId *id = g_new0 (ECalComponentId, 1);
+	ECalBackendExchangeComponent *ecomp;
 
+	ecomp = (ECalBackendExchangeComponent *)value;
+	
+	/* FIXME Need get the recurrence id here */
+	id->uid = g_strdup (key);
+	id->rid = NULL;
+	if (ecomp->icomp) {
+		/* FIXME somehow the query does not match with the component in some cases, so user needs to press a
+		   clear to get rid of the component from the view in that case*/
+		e_cal_backend_notify_object_removed (backend, id, icalcomponent_as_ical_string (ecomp->icomp)
+				, NULL);
+	}
 	g_hash_table_remove (cbex->priv->objects, key);
+	e_cal_component_free_id (id);
 }
 
 void
@@ -740,8 +792,7 @@ get_object (ECalBackendSync *backend, EDataCal *cal,
 				struct icaltimetype itt;
 
 				itt = icaltime_from_string (rid);
-				new_inst = e_cal_util_construct_instance (
-					icalcomponent_new_clone (ecomp->icomp), itt);
+				new_inst = e_cal_util_construct_instance (ecomp->icomp, itt);
 				if (!new_inst)
 					return GNOME_Evolution_Calendar_ObjectNotFound;
 				
@@ -1091,7 +1142,7 @@ match_object (gpointer key, gpointer value, gpointer data)
 	ECalBackendExchangeComponent *ecomp = value;
 	struct search_data *sd = data;
 	ECalComponent *cal_comp;
-	icalcomponent *tmp;
+	icalcomponent *tmp = NULL;
 	char * ecal_str;
 	GList *inst;
 
@@ -1099,8 +1150,9 @@ match_object (gpointer key, gpointer value, gpointer data)
 	cal_comp = e_cal_component_new ();
 
 	/* Find a way to test if the icalcomp added to cal_comp is not null */
-	tmp = icalcomponent_new_clone (ecomp->icomp);
-	if (tmp) {
+	/* ecomp->icomp would not exist for a detached instance with no master object */
+	if (ecomp->icomp) {
+		tmp = icalcomponent_new_clone (ecomp->icomp);
 		e_cal_component_set_icalcomponent (cal_comp, tmp);
 		if (e_cal_backend_sexp_match_comp (sd->sexp, cal_comp, sd->backend)) {
 			ecal_str = e_cal_component_get_as_string (cal_comp);
@@ -1160,6 +1212,18 @@ e_cal_backend_exchange_is_online (ECalBackendExchange *cbex)
 		return TRUE;
 }
 
+static gboolean
+is_loaded (ECalBackend *backend)
+{
+	ECalBackendExchange *cbex;
+	ECalBackendExchangePrivate *priv;
+
+	cbex = E_CAL_BACKEND_EXCHANGE (backend);
+	priv = cbex->priv;
+
+	return priv->is_loaded;
+}
+
 static CalMode
 get_mode (ECalBackend *backend)
 {
@@ -1179,9 +1243,6 @@ set_mode (ECalBackend *backend, CalMode mode)
 {
 	ECalBackendExchange *cbex;
 	ECalBackendExchangePrivate *priv;
-	ExchangeAccount *account;
-	ExchangeAccountResult acresult;
-	const char *uristr;
 	
 	cbex = E_CAL_BACKEND_EXCHANGE (backend);
 	priv = cbex->priv;
@@ -1199,44 +1260,26 @@ set_mode (ECalBackend *backend, CalMode mode)
 	switch (mode) {
 	
 	case CAL_MODE_REMOTE:
-			/* Change status to be online now */
-			/* Should we check for access rights before setting this ? */
-			d(printf ("set mode to online\n"));
-			account = exchange_component_get_account_for_uri (global_exchange_component, NULL);
-			/* check if authentication is required */
-			if (!exchange_account_get_context (account))
-				e_cal_backend_notify_auth_required(backend);
-
-			uristr = e_cal_backend_get_uri (E_CAL_BACKEND (backend));
-			account = exchange_component_get_account_for_uri (global_exchange_component, uristr);
-			if (account)
-				cbex->folder = exchange_account_get_folder (account, uristr);
-
 			e_cal_backend_notify_mode (backend, 
 				GNOME_Evolution_Calendar_CalListener_MODE_SET,
 				GNOME_Evolution_Calendar_MODE_REMOTE);
 			/* FIXME : Test if available for read already */
 			priv->read_only = FALSE;
 			priv->mode = CAL_MODE_REMOTE;
+		
+			e_cal_backend_notify_readonly (backend, priv->read_only);
+			if (is_loaded (backend))
+				e_cal_backend_notify_auth_required(backend);
 			break;
 
 	case CAL_MODE_LOCAL:
 			d(printf ("set mode to offline\n"));
-			account = exchange_component_get_account_for_uri (global_exchange_component, NULL);
-			if (account) {
-				/* Call exchange_account_connect to populate the folder hierarchy,
-				 * so that load_cache succeeds */
-				exchange_account_set_offline (account);
-				if (!exchange_account_connect (account, NULL, &acresult)) {
-					uristr = e_cal_backend_get_uri (E_CAL_BACKEND (backend));
-					cbex->folder = exchange_account_get_folder (account, uristr);
-				}
-			}
-			priv->mode = CAL_MODE_LOCAL;
+					priv->mode = CAL_MODE_LOCAL;
 			priv->read_only = TRUE;
 			e_cal_backend_notify_mode (backend, 
 				GNOME_Evolution_Calendar_CalListener_MODE_SET,
 				GNOME_Evolution_Calendar_MODE_LOCAL);
+			e_cal_backend_notify_readonly (backend, priv->read_only);
 			break;
 
 	default :
@@ -1355,11 +1398,13 @@ e_cal_backend_exchange_make_timestamp_rfc822 (time_t when)
 char *
 e_cal_backend_exchange_get_from_string (ECalBackendSync *backend, ECalComponent *comp)
 {
-	char *name, *addr;
+	char *name = NULL, *addr = NULL, *from_string = NULL;
 
 	e_cal_backend_exchange_get_from (backend, comp, &name, &addr);
-	
-	return g_strdup_printf ("\"%s\" <%s>", name, addr);
+	from_string = g_strdup_printf ("\"%s\" <%s>", name, addr);
+	g_free (name);
+	g_free (addr);
+	return from_string;
 }
 
 gchar *
@@ -1423,8 +1468,8 @@ check_change_type (gpointer key, gpointer value, gpointer data)
 		}
 		
 		g_free (calobj);
+		g_object_unref (comp);		
 	}
-	g_object_unref (comp);		
 }
 
 struct cbe_data {
@@ -1527,9 +1572,10 @@ get_attachment (ECalBackendExchange *cbex, const char *uid,
 				attach_file = g_strdup_printf ("%s/%s-%s", cbex->priv->local_attachment_store, uid, filename);
 				// Attach
 				attach_file_url = save_attach_file (attach_file, attach_data, stream_mem->buffer->len);
+				g_free (attach_data);
 				g_free (attach_file);
 				d(printf ("attach file name : %s\n", attach_file_url));
-				list = g_slist_append (list, g_strdup (attach_file_url));
+				list = g_slist_append (list, attach_file_url);
 
 				camel_object_unref (stream);
 			}
@@ -1589,7 +1635,7 @@ build_msg ( ECalBackendExchange *cbex, ECalComponent *comp, const char *subject,
 	CamelContentType *type;
 	const char *uid;
 	char *buffer = NULL, *cid;
-	char *from_name, *from_email;
+	char *from_name = NULL, *from_email = NULL;
 	GSList *attach_list = NULL, *l, *new_attach_list = NULL;
 	char *fname, *file_contents = NULL, *filename, *dest_url, *mime_filename, *attach_file;
 	int len = 0;
@@ -1606,6 +1652,8 @@ build_msg ( ECalBackendExchange *cbex, ECalComponent *comp, const char *subject,
 	from = camel_internet_address_new ();
 	camel_internet_address_add (from, from_name, from_email);
 	camel_mime_message_set_from (msg, from);
+	g_free (from_name);
+	g_free (from_email);
 	camel_object_unref (from);
 
 	e_cal_component_get_uid (comp, &uid);
@@ -1658,7 +1706,7 @@ build_msg ( ECalBackendExchange *cbex, ECalComponent *comp, const char *subject,
 		camel_multipart_add_part (multipart, mime_part);
 		camel_object_unref (mime_part);
 		g_free (cid);
-
+		g_free (dest_url);
 	}
 	if (!new_attach_list) {
 		camel_object_unref (multipart);
@@ -1793,6 +1841,7 @@ init (ECalBackendExchange *cbex)
 
 	cbex->priv->set_lock = g_mutex_new ();
 	cbex->priv->open_lock = g_mutex_new ();
+	cbex->priv->cache_unseen = NULL;
 
 	e_cal_backend_sync_set_lock (E_CAL_BACKEND_SYNC (cbex), TRUE);
 }
@@ -1818,6 +1867,7 @@ finalize (GObject *object)
 		g_hash_table_destroy (cbex->priv->cache_unseen);
 	g_free (cbex->priv->object_cache_file);
 	g_free (cbex->priv->lastmod);
+	g_free (cbex->priv->local_attachment_store);
 
 	g_hash_table_destroy (cbex->priv->timezones);
 
@@ -1871,6 +1921,7 @@ class_init (ECalBackendExchangeClass *klass)
  	backend_class->start_query = start_query;
  	backend_class->get_mode = get_mode;
  	backend_class->set_mode = set_mode;
+	backend_class->is_loaded = is_loaded;
 	backend_class->internal_get_default_timezone = internal_get_default_timezone;
 	backend_class->internal_get_timezone = internal_get_timezone;
 
